%define qemu_name	qemu-kvm
%define qemu_version	0.14.0
%define qemu_rel	3
%define qemu_release	%mkrel %{?qemu_snapshot:0.%{qemu_snapshot}.}%{qemu_rel}

%define __find_requires %{_builddir}/%{qemu_name}-%{qemu_version}%{?qemu_snapshot:-%{qemu_snapshot}}/find_requires.sh

Summary:	QEMU CPU Emulator
Name:		qemu
Version:	%{qemu_version}
Release:	%{qemu_release}
Source0:	http://kent.dl.sourceforge.net/sourceforge/kvm/%{qemu_name}-%{version}%{?qemu_snapshot:-%{qemu_snapshot}}.tar.xz
Source1:	kvm.modules
Patch0:		qemu-kvm-compile-fix.patch
# KSM control scripts
Source4: ksm.init
Source5: ksm.sysconfig
Source6: ksmtuned.init
Source7: ksmtuned
Source8: ksmtuned.conf

License:	GPL
URL:		http://bellard.org/qemu/
Group:		Emulators
Provides:	kvm
Obsoletes:	kvm < 86
Requires:	qemu-img = %{version}-%{release}
Requires:	cyrus-sasl
Requires:   	glibc >= 2.12.1
Requires:   	libstdc++6 >= 4.5.2
BuildRequires:	libSDL-devel
BuildRequires:	tetex-texi2html
BuildRequires:	texinfo
BuildRequires:	e2fsprogs-devel
BuildRequires:  kernel-headers	
BuildRequires:	pulseaudio-devel
BuildRequires:	zlib-devel
BuildRequires:	brlapi-devel
BuildRequires:	gnutls-devel
BuildRequires:	libsasl2-devel
BuildRequires:	pciutils-devel
BuildRequires:	dev86
BuildRequires:	iasl
BuildRequires:	glibc-devel >= 2.10.1
ExclusiveArch:	%{ix86} ppc x86_64 amd64 %{sunsparc}
BuildRoot:	%{_tmppath}/%{name}-%{version}-%{release}-buildroot

%description
QEMU is a FAST! processor emulator. By using dynamic translation it
achieves a reasonnable speed while being easy to port on new host
CPUs. QEMU has two operating modes:

* User mode emulation. In this mode, QEMU can launch Linux processes
  compiled for one CPU on another CPU. Linux system calls are
  converted because of endianness and 32/64 bit mismatches. Wine
  (Windows emulation) and DOSEMU (DOS emulation) are the main targets
  for QEMU.

* Full system emulation. In this mode, QEMU emulates a full system,
  including a processor and various peripherials. Currently, it is
  only used to launch an x86 Linux kernel on an x86 Linux system. It
  enables easier testing and debugging of system code. It can also be
  used to provide virtual hosting of several virtual PC on a single
  server.

%description -l de
QEMU ist ein SCHNELLER! Prozessor Emulator. Unter Verwendung von dynamischen Übersetzungen. 
Er erreicht eine akzeptable Geschwindigkeit Eine akzeptable Geschwindigkeit wird beim einfachen 
portieren auf neue Host Prozessoren erreicht. QEMU verfügt über zwei Betriebssystem Modi:

* Emulation im Benutzermodus. Bei diesem Modus kann QEMU Linux-Prozesse für eine CPU auf einem 
  anderen CPU kompiliert starten. Linux-Systemaufrufe werden aufgrund der Byte-Reihenfolge und 
  32/64-Bit Unterschiede konvertiert. Wine (Windows-Emulation) und DOSEMU (DOS-Emulation) sind 
  die wichtigsten Ziele für QEMU.

* Emulation des vollständigen Systems. Bei diesem Modus emuliert QEMU ein vollständiges System, 
  inklusive eines Prozessors und verschiedener Peripherie. Aktuell wird es nur verwendet, um einen 
  x 86-Linux Kernel auf einem x 86 Linux-System zu starten. Es ermöglicht einfaches Testen und 
  Fehlerbehebung von System-Code. Sie können auch zur virtuellen Hosten von mehreren virtuellen PC's 
  auf einem einzelnen Server verwendet werden. 

%package img
Summary:	QEMU disk image utility
Summary(de):	QEMU Festplatten-Abbild-Programm
Group:		Emulators
Version:	%{qemu_version}
Release:	%{qemu_release}

%description img
This package contains the QEMU disk image utility that is used to
create, commit, convert and get information from a disk image.

%description img -l de
Dieses Paket enthält das QEMU Festplatten-Abbild-Programm, das zum Erstellen, festlegen, 
konvertieren und erhalten von Informationen von einem Datenträgerabbild verwendet wird. 

%prep
%setup -q -n %{qemu_name}-%{qemu_version}%{?qemu_snapshot:-%{qemu_snapshot}}
%patch0 -p0

cat >find_requires.sh << EOF
#!/bin/sh
%{_prefix}/lib/rpm/find-requires %{buildroot} %{_target_cpu} | grep -v GLIBC_PRIVATE
EOF
chmod +x find_requires.sh

%build
if ! echo | %{__cc} -mtune=generic -xc -c - -o /dev/null 2> /dev/null; then
  CFLAGS=`echo "$RPM_OPT_FLAGS" | sed -e "s/-mtune=generic/-mtune=pentiumpro/g"`
fi
 
extraldflags="-Wl,--build-id";
buildldflags="VL_LDFLAGS=-Wl,--build-id"

%ifarch %{ix86} x86_64
./configure --target-list="i386-softmmu x86_64-softmmu i386-linux-user x86_64-linux-user" \
            --prefix=%{_prefix} \
	    --sysconfdir=%{_sysconfdir} \
            --audio-drv-list=pa,sdl,alsa,oss \
            --extra-ldflags=$extraldflags \
            --disable-strip \
            --disable-xen \
%ifarch x86_64
            --enable-spice \
%endif
            --extra-cflags="$CFLAGS -march=i586"


%make V=1 $buildldflags
cp -a x86_64-softmmu/qemu-system-x86_64 qemu-kvm
make clean
%endif

./configure \
	--target-list="i386-softmmu x86_64-softmmu arm-softmmu m68k-softmmu \
           mips-softmmu mipsel-softmmu mips64-softmmu mips64el-softmmu ppc-softmmu \
           ppcemb-softmmu ppc64-softmmu sh4-softmmu sh4eb-softmmu \
           i386-linux-user x86_64-linux-user alpha-linux-user arm-linux-user \
           armeb-linux-user cris-linux-user m68k-linux-user mips-linux-user \
           mipsel-linux-user ppc-linux-user ppc64-linux-user ppc64abi32-linux-user \
           sh4-linux-user sh4eb-linux-user sparc-linux-user sparc64-linux-user \
           sparc32plus-linux-user" \
	--prefix=%{_prefix} \
	--sysconfdir=%{_sysconfdir} \
	--interp-prefix=%{_prefix}/qemu-%%M \
	--audio-drv-list=pa,sdl,alsa,oss \
	--disable-kvm \
    	--disable-strip \
        --disable-xen \
        --disable-werror \
	--extra-ldflags=$extraldflags \
%ifarch x86_64
            --enable-spice \
%endif
	--extra-cflags="$CFLAGS"

%make V=1 $buildldflags

%install
rm -rf $RPM_BUILD_ROOT

install -D -p -m 0755 %{SOURCE4} $RPM_BUILD_ROOT/etc/rc.d/init.d/ksm
install -D -p -m 0644 %{SOURCE5} $RPM_BUILD_ROOT%{_sysconfdir}/sysconfig/ksm

install -D -p -m 0755 %{SOURCE6} $RPM_BUILD_ROOT/etc/rc.d/init.d/ksmtuned
install -D -p -m 0755 %{SOURCE7} $RPM_BUILD_ROOT%{_sbindir}/ksmtuned
install -D -p -m 0644 %{SOURCE8} $RPM_BUILD_ROOT%{_sysconfdir}/ksmtuned.conf

%ifarch %{ix86} x86_64
mkdir -p $RPM_BUILD_ROOT/%{_sysconfdir}/sysconfig/modules
mkdir -p $RPM_BUILD_ROOT%{_bindir}/
mkdir -p $RPM_BUILD_ROOT%{_datadir}/%{name}

install -m 0755 %{SOURCE1} $RPM_BUILD_ROOT/%{_sysconfdir}/sysconfig/modules/kvm.modules
install -m 0755 kvm/kvm_stat $RPM_BUILD_ROOT%{_bindir}/
install -m 0755 qemu-kvm $RPM_BUILD_ROOT%{_bindir}/
%endif

%makeinstall_std BUILD_DOCS="yes"
install -D -p -m 0644 qemu.sasl $RPM_BUILD_ROOT%{_sysconfdir}/sasl2/qemu.conf

# remove unpackaged files
rm -rf $RPM_BUILD_ROOT%{_docdir}/qemu
rm -rf ${RPM_BUILD_ROOT}%{_datadir}/%{name}/s390-zipl.rom
rm -rf ${RPM_BUILD_ROOT}%{_datadir}/%{name}/gpxe*rom

%clean
rm -rf $RPM_BUILD_ROOT

%post 
%ifarch %{ix86} x86_64
# load kvm modules now, so we can make sure no reboot is needed.
# If there's already a kvm module installed, we don't mess with it
sh /%{_sysconfdir}/sysconfig/modules/kvm.modules
%endif
%_post_service ksmtuned
%_post_service ksm

%preun
%_preun_service ksm
%_preun_service ksmtuned

%triggerpostun -- qemu < 0.10.4-6
rm -f /etc/rc.d/*/{K,S}??qemu

%files
%defattr(-,root,root)
%doc README qemu-doc.html qemu-tech.html
%config(noreplace)%{_sysconfdir}/sasl2/qemu.conf
/etc/rc.d/init.d/ksm
%config(noreplace) %{_sysconfdir}/sysconfig/ksm
/etc/rc.d/init.d/ksmtuned
%{_sbindir}/ksmtuned
%config(noreplace) %{_sysconfdir}/ksmtuned.conf
%{_sysconfdir}/sysconfig/modules/kvm.modules
%{_sysconfdir}/qemu/target-x86_64.conf
%{_bindir}/kvm_stat
%{_bindir}/qemu-io
%{_bindir}/qemu-kvm
%{_bindir}/qemu
%{_bindir}/qemu-alpha
%{_bindir}/qemu-arm*
%{_bindir}/qemu-cris
%{_bindir}/qemu-i386
%{_bindir}/qemu-m68k
%{_bindir}/qemu-mips*
%{_bindir}/qemu-nbd
%{_bindir}/qemu-ppc*
%{_bindir}/qemu-sh4*
%{_bindir}/qemu-sparc*
%{_bindir}/qemu-x86_64
%{_bindir}/qemu-system-arm
%{_bindir}/qemu-system-m68k
%{_bindir}/qemu-system-sh4*
%{_bindir}/qemu-system-ppc*
%{_bindir}/qemu-system-mips*
%{_bindir}/qemu-system-x86_64
%{_mandir}/man1/qemu.1*
%{_mandir}/man8/qemu-nbd.8*
%dir %{_datadir}/qemu
%dir %{_datadir}/qemu/keymaps/
%{_datadir}/qemu/*.bin
%{_datadir}/qemu/keymaps
%{_datadir}/qemu/openbios-sparc32
%{_datadir}/qemu/openbios-sparc64
%{_datadir}/qemu/openbios-ppc
%{_datadir}/qemu/bamboo.dtb
%{_datadir}/qemu/petalogix-s3adsp1800.dtb

%files img
%defattr(-,root,root)
%{_bindir}/qemu-img
%{_mandir}/man1/qemu-img.1*


%changelog
* Thu Mar 31 2011 Texstar <texstar at gmail.com> 0.14.0-3pclos2011
- add qemu-ksm-error-suppressed.patch to ksm init

* Tue Mar 01 2011 maik3531 <maik3531 at yahoo.de> 0.14.0-2pclos2010
- remove *.rom 
- disable-strip, disable-xen, disable-werror

* Wed Feb 23 2011 maik3531 <maik3531 at yahoo.de> 0.14.0-1pclos2010
- add german description
- add texinfo to buildrequires
- add compile-fix.patch from mdv
- add -march=i586 to cflags show https://bugzilla.redhat.com/show_bug.cgi?id=624279
- fix target-list
- fix the sysconfdir param
- disable the kvm_trace tool
- try the official doc/man install
- disable qemu-cris and qemu-sparc due to build errors

* Fri Jul 16 2010 Texstar <texstar at gmail.com> 0.12.4-1pclos2010
- 0.12.4

* Fri Jan 22 2010 Texstar <texstar at gmail.com> 0.12.2-1pclos2010
- 0.12.2

